@@ -2,7 +2,7 @@
 zimsm.fit.control <- function(
   predmethod=c("components", "catprobs")
 ){
-  #' @title control of fitting parameters for zero inflated MSMs
+  #' @title Control of fitting parameters for zero inflated MSMs
   #' @description this is an internal function called by 
   #'  \code{\link[qgcomp]{qgcomp.zi.boot}},
   #'  but is documented here for clarity. Generally, users will not need to call
@@ -30,14 +30,14 @@ zimsm.fit <- function(
   main=TRUE, 
   degree=1, 
   id=NULL,
-  weights=NULL,
+  weights,
   MCsize=10000, 
   containmix=list(count=TRUE, zero=TRUE),
   bayes=FALSE,
   x=FALSE,
   msmcontrol=zimsm.fit.control(),
   ...){
-  #' @title secondary prediction method for the (zero-inflated) qgcomp MSM.
+  #' @title Secondary prediction method for the (zero-inflated) qgcomp MSM.
   #' @description this is an internal function called by 
   #'  \code{\link[qgcomp]{qgcomp.zi.boot}},
   #'  but is documented here for clarity. Generally, users will not need to call
@@ -95,14 +95,32 @@ zimsm.fit <- function(
   #'   x=FALSE)
   #' msmfit$msmfit
   #' }
-  XXweights <- NULL
+
+  newform <- terms(f, data = qdata)
+  class(newform) <- "formula"
+
+  nobs = nrow(qdata)
+  origcall <- thecall <- match.call(expand.dots = FALSE)
+  names(thecall) <- gsub("f", "formula", names(thecall))
+  names(thecall) <- gsub("qdata", "data", names(thecall))
+  m <- match(c("formula", "data", "weights", "offset"), names(thecall), 0L)
+  hasweights = ifelse("weights" %in% names(thecall), TRUE, FALSE)
+  thecall <- thecall[c(1L, m)]
+  thecall$drop.unused.levels <- TRUE
+  
+  thecall[[1L]] <- quote(stats::model.frame)
+  thecalle <- eval(thecall, parent.frame())
+  if(hasweights){
+    qdata$weights <- as.vector(model.weights(thecalle))
+  } else qdata$weights = rep(1, nobs)
+
   if(is.null(id)) {
     id = "id__"
     qdata$id__ = 1:dim(qdata)[1]
   }
   # conditional outcome regression fit
-  if(!bayes) fit <- zeroinfl(f, data = qdata[,!(names(qdata) %in% id), drop=FALSE],
-                             #weights=weights, 
+  if(!bayes) fit <- zeroinfl(newform, data = qdata[,!(names(qdata) %in% id), drop=FALSE],
+                             weights=weights, 
                              ...)
   if(bayes) stop("Bayes not yet implemented for this function")
   if(fit$optim$convergence[1]!=0) warning("Conditional outcome regression model did not converge")
@@ -135,17 +153,19 @@ zimsm.fit <- function(
     newY
   }
   newids <- data.frame(temp=sort(sample(unique(qdata[,id, drop=TRUE]), MCsize, 
-                                        #probs=weights, #bootstrap sampling with weights works with fixed weights, but not time-varying weights
                                         replace = TRUE
   )))
   names(newids) <- id
   newdata <- merge(qdata,newids, by=id, all.x=FALSE, all.y=TRUE)[1:MCsize,]
-  predmat = lapply(intvals, predit, newdata=newdata)
-  newdata = NULL
+  predmat <- lapply(intvals, predit, newdata=newdata)
   msmdat <- data.frame(
-    Ya = do.call("c", predmat),
-    psi = rep(intvals, each=MCsize)
+   # weights = rep(newdata$weights, times=length(table(qdata[expnms[1]])))
+    weights = rep(newdata$weights, times=length(intvals))
   )
+  newdata = NULL
+  msmdat$Ya = do.call("c", predmat)
+  msmdat$psi = rep(intvals, each=MCsize)
+  
   fstr = paste("Ya ~ 1", 
                ifelse(containmix[["count"]], "+ poly(psi, degree=degree, raw=TRUE) | 1", "| 1"),
                ifelse(containmix[["zero"]], "+ poly(psi, degree=degree, raw=TRUE)", "")
@@ -153,12 +173,9 @@ zimsm.fit <- function(
   #if(!is.null(weights)){
   #  msmdat[,'__weights'] = newdata[,weights]
   #}
-  if(is.null(weights)){
-    msmdat[,'XXweights'] = 1
-  }
 
   msmfit <- zeroinfl(as.formula(fstr), data=msmdat, x=x,
-                     weights=XXweights,
+                     weights=weights,
                      ...)
   if(msmfit$optim$convergence[1]!=0) warning("MSM did not converge")
   
@@ -181,10 +198,10 @@ qgcomp.zi.noboot <- function(f,
                              q=4, 
                              breaks=NULL, 
                              id=NULL,
-                             weights=NULL,
+                             weights,
                              alpha=0.05, 
                              bayes=FALSE, ...){
-  #' @title quantile g-computation for zero-inflated count outcomes under linearity/additivity
+  #' @title Quantile g-computation for zero-inflated count outcomes under linearity/additivity
   #'
   #' @description This function estimates a linear dose-response parameter representing a one quantile
   #' increase in a set of exposures of interest for zero-inflated count outcomes. This function is 
@@ -216,12 +233,13 @@ qgcomp.zi.noboot <- function(f,
   #' @param id (optional) NULL, or variable name indexing individual units of 
   #' observation (only needed if analyzing data with multiple observations per 
   #' id/cluster)
-  #' @param weights not yet implemented
+  #' @param weights "case weights" - passed to the "weight" argument of 
+  #' \code{\link[pscl]{zeroinfl}}.
   #' @param alpha alpha level for confidence limit calculation
   #' @param bayes not yet implemented
   #' @param ... arguments to zeroinfl (e.g. dist)
-  #' @seealso \code{\link[qgcomp]{qgcomp.noboot}}, \code{\link[qgcomp]{qgcomp.cox.noboot}}, 
-  #'  and \code{\link[pscl]{zeroinfl}}
+  #' @seealso \code{\link[qgcomp]{qgcomp.zi.boot}},\code{\link[qgcomp]{qgcomp.noboot}}, 
+  #' \code{\link[qgcomp]{qgcomp.cox.noboot}},  and \code{\link[pscl]{zeroinfl}}
   #' @return a qgcompfit object, which contains information about the effect
   #'  measure of interest (psi) and associated variance (var.psi), as well
   #'  as information on the model fit (fit) and information on the 
@@ -234,25 +252,59 @@ qgcomp.zi.noboot <- function(f,
   #' set.seed(50)
   #' n=100
   #' dat <- data.frame(y=rbinom(n, 1, 0.5)*rpois(n, 1.2), x1=runif(n), x2=runif(n), z=runif(n))
+  #' 
   #' # poisson count model, mixture in both portions
   #' qgcomp.zi.noboot(f=y ~ z + x1 + x2 | x1 + x2, expnms = c('x1', 'x2'), 
   #'     data=dat, q=2, dist="poisson")
+  #'     
   #' # negative binomial count model, mixture and covariate in both portions
   #' qgcomp.zi.noboot(f=y ~ z + x1 + x2 | z + x1 + x2, expnms = c('x1', 'x2'), 
   #'    data=dat, q=2, dist="negbin")  
   #' qgcomp.zi.noboot(f=y ~ z + x1 + x2, expnms = c('x1', 'x2'), 
   #'    data=dat, q=2, dist="negbin") # equivalent
+  #'    
   #' # negative binomial count model, mixture only in the 'count' portion of the model
   #' qgcomp.zi.noboot(f=y ~ z + x1 + x2 | z, expnms = c('x1', 'x2'), data=dat, q=2, dist="negbin")
-  
+  #' 
+  #' # weighted analysis
+  #' dat$w = runif(n)*5
+  #' qgcomp.zi.noboot(f=y ~ z + x1 + x2 | x1 + x2, expnms = c('x1', 'x2'), 
+  #'     data=dat, q=2, dist="poisson", weights=w)
+  #' # Expect this:     
+  #' # Warning message:
+  #' # In eval(family$initialize) : non-integer #successes in a binomial glm!
+  #' 
+
   # list containers
   estb <- vcov_mod <- seb <- tstat <- pvalz <- allterms <- containmix <- pos.weights <- neg.weights <- 
     pos.coef <- neg.coef <- pos.psi <- neg.psi <- pos.size <- neg.size <- wcoef <- ci <- tstat <- list()
   suppressWarnings(testfit <- zeroinfl(f, data = data, msmcontrol=zeroinfl.control(maxit = 1, EM=FALSE)))
   allterms$count = attr(terms(testfit, "count"), "term.labels")
   allterms$zero = attr(terms(testfit, "zero"), "term.labels")
+
+  newform <- terms(f, data = data)
+  class(newform) <- "formula"
+
+  nobs = nrow(data)
+  origcall <- thecall <- match.call(expand.dots = FALSE)
+  names(thecall) <- gsub("f", "formula", names(thecall))
+  m <- match(c("formula", "data", "weights", "offset"), names(thecall), 0L)
+  hasweights = ifelse("weights" %in% names(thecall), TRUE, FALSE)
+  thecall <- thecall[c(1L, m)]
+  thecall$drop.unused.levels <- TRUE
+  
+  thecall[[1L]] <- quote(stats::model.frame)
+  thecalle <- eval(thecall, parent.frame())
+  if(hasweights){
+    data$weights <- as.vector(model.weights(thecalle))
+  } else data$weights = rep(1, nobs)
+
+
   if (is.null(expnms)) {
-    expnms <- attr(terms(testfit), "term.labels")
+  
+    #expnms <- attr(terms(testfit), "term.labels")
+    expnms <- attr(newform, "term.labels")
+  
     message("Including all model terms as exposures of interest (count and zero parts must be identical)\n")      
   }
   lin = checknames(expnms)
@@ -278,8 +330,8 @@ qgcomp.zi.noboot <- function(f,
   }
   
 
-  if(!bayes) fit <- zeroinfl(f, data = qdata[,!(names(qdata) %in% id), drop=FALSE], 
-                             #weights=weights, 
+  if(!bayes) fit <- zeroinfl(newform, data = qdata, 
+                             weights=weights, 
                              ...)
   if(bayes){
     stop("bayesian zero inflated models not yet implemented")
@@ -342,7 +394,7 @@ qgcomp.zi.noboot <- function(f,
     neg.size = neg.size,
     bootstrap=FALSE,
     cov.yhat=NULL,
-    alpha=alpha
+    alpha=alpha, call=origcall
   )
   #if(fit$family$family=='gaussian'){
   #  res$tstat <- tstat
@@ -363,7 +415,7 @@ qgcomp.zi.boot <- function(f,
                            q=4, 
                            breaks=NULL, 
                            id=NULL,
-                           weights=NULL,
+                           weights,
                            alpha=0.05, 
                            B=200, 
                            degree=1, 
@@ -373,7 +425,7 @@ qgcomp.zi.boot <- function(f,
                            MCsize=10000, 
                            msmcontrol=zimsm.fit.control(),
                           ...){
-  #' @title quantile g-computation for zero-inflated count outcomes
+  #' @title Quantile g-computation for zero-inflated count outcomes
   #'  
   #' @description This function estimates a linear dose-response parameter representing a one quantile
   #' increase in a set of exposures of interest for zero-inflated count outcomes. This function is 
@@ -428,7 +480,8 @@ qgcomp.zi.boot <- function(f,
   #' @param id (optional) NULL, or variable name indexing individual units of 
   #' observation (only needed if analyzing data with multiple observations per 
   #' id/cluster)
-  #' @param weights not yet implemented
+  #' @param weights "case weights" - passed to the "weight" argument of 
+  #' \code{\link[pscl]{zeroinfl}}. NOTE - this does not work with parallel=TRUE!
   #' @param alpha alpha level for confidence limit calculation
   #' @param B integer: number of bootstrap iterations (this should typically be
   #' >=200, though it is set lower in examples to improve run-time).
@@ -445,7 +498,8 @@ qgcomp.zi.boot <- function(f,
   #'  error at a given value of MCsize)
   #' @param msmcontrol named list from \code{\link[qgcomp]{zimsm.fit.control}}
   #' @param ... arguments to glm (e.g. family)
-  #' @seealso \code{\link[qgcomp]{qgcomp.noboot}}, and \code{\link[qgcomp]{qgcomp}}
+  #' @seealso \code{\link[qgcomp]{qgcomp.zi.noboot}},\code{\link[qgcomp]{qgcomp.boot}}, 
+  #' \code{\link[qgcomp]{qgcomp.cox.boot}},  and \code{\link[pscl]{zeroinfl}}
   #' @return a qgcompfit object, which contains information about the effect
   #'  measure of interest (psi) and associated variance (var.psi), as well
   #'  as information on the model fit (fit) and information on the 
@@ -477,6 +531,26 @@ qgcomp.zi.boot <- function(f,
   #' # negative binomial count model, mixture and covariate in both portions
   #' qgcomp.zi.boot(f=y ~ z + x1 + x2 | z + x1 + x2, expnms = c('x1', 'x2'), 
   #'    data=dat, q=4, dist="negbin", B=10, MCsize=10000) 
+  #'    
+  #' # weighted analysis (NOTE THIS DOES NOT WORK WITH parallel=TRUE!)
+  #' dat$w = runif(n)*5
+  #' qgcomp.zi.noboot(f=y ~ z + x1 + x2 | x1 + x2, expnms = c('x1', 'x2'), 
+  #'     data=dat, q=4, dist="poisson", weights=w)
+  #' # Expect this:     
+  #' # Warning message:
+  #' # In eval(family$initialize) : non-integer #successes in a binomial glm!
+  #' qgcomp.zi.boot(f=y ~ x1 + x2 | x1 + x2, expnms = c('x1', 'x2'), 
+  #'     data=dat, q=4, dist="poisson", B=5, MCsize=50000, parallel=FALSE, weights=w)
+
+  #' # Log rr per one IQR change in all exposures (not on quantile basis)
+  #' dat$x1iqr <- dat$x1/with(dat, diff(quantile(x1, c(.25, .75))))
+  #' dat$x2iqr <- dat$x2/with(dat, diff(quantile(x2, c(.25, .75))))
+  #' # note that I(x>...) now operates on the untransformed value of x,
+  #' # rather than the quantized value
+  #' res2 = qgcomp.zi.boot(y ~ z + x1iqr + x2iqr + I(x2iqr>0.1) + I(x2>0.4) + I(x2>0.9) | x1iqr + x2iqr, 
+  #'                    family="binomial", expnms = c('x1iqr', 'x2iqr'), data=dat, q=NULL, B=2, 
+  #'                    degree=2, MCsize=200, dist="poisson")
+  #' res2
   #' }
   if(is.null(seed)) seed = round(runif(1, min=0, max=1e8))
   message("qgcomp.zi.boot function is still experimental. Please use with caution and be sure results are reasonable.\n")      
@@ -486,15 +560,46 @@ qgcomp.zi.boot <- function(f,
   suppressWarnings(testfit <- zeroinfl(f, data = data, control=zeroinfl.control(maxit = 1, EM=FALSE)))
   allterms$count = attr(terms(testfit, "count"), "term.labels")
   allterms$zero = attr(terms(testfit, "zero"), "term.labels")
+
+  newform <- terms(f, data = data)
+  class(newform) <- "formula"
+
+  nobs = nrow(data)
+  origcall <- thecall <- match.call(expand.dots = FALSE)
+  names(thecall) <- gsub("f", "formula", names(thecall))
+  m <- match(c("formula", "data", "weights", "offset"), names(thecall), 0L)
+  hasweights = ifelse("weights" %in% names(thecall), TRUE, FALSE)
+  thecall <- thecall[c(1L, m)]
+  thecall$drop.unused.levels <- TRUE
+  
+  thecall[[1L]] <- quote(stats::model.frame)
+  thecalle <- eval(thecall, parent.frame())
+  if(hasweights){
+    data$weights <- as.vector(model.weights(thecalle))
+  } else data$weights = rep(1, nobs)
+
+
   if (is.null(expnms)) {
-    expnms <- attr(terms(testfit), "term.labels")
+  
+    #expnms <- attr(terms(testfit), "term.labels")
+    expnms <- attr(newform, "term.labels")
+  
     message("Including all model terms as exposures of interest (count and zero parts must be identical)\n")      
   }
+  expterms = match(all.vars(f, unique=FALSE), expnms, nomatch = -99)
+  expterms = expterms[expterms>0]
+  if((length(expterms) != 2*length(expnms) & length(expterms) != length(expnms))){
+    warning("Ensure that all of the 
+    variables in 'expnms' are in either the count model, the zero model, or both,
+    and that neither model contains only a subset of exposures (this can sometimes be on purpose but will often cause errors)."
+    )
+    
+  }
   for(modtype in c("count", "zero")){
     containmix[[modtype]] = all(expnms %in% allterms[[modtype]])
-    if (!containmix[[modtype]] & any(expnms %in% allterms[[modtype]])) stop("Ensure that all of the 
-    variables in 'expnms' are in either the count model, the zero model, or both,
-    and that neither model contains only a subset of exposures.")
+  #  if (!containmix[[modtype]] & any(expnms %in% allterms[[modtype]])) warning("Ensure that all of the 
+  #  variables in 'expnms' are in either the count model, the zero model, or both,
+  #  and that neither model contains only a subset of exposures (this can sometimes be on purpose but will often cause errors).")
   }
   #lin = checknames(expnms)
   #if(!lin) stop("Model appears to be non-linear and I'm having trouble parsing it: 
@@ -531,10 +636,9 @@ qgcomp.zi.boot <- function(f,
     qdata$id__ <- 1:dim(qdata)[1]
   }
   ###
-  msmfit <- zimsm.fit(f, qdata, intvals, expnms, main=TRUE,
+  msmfit <- zimsm.fit(newform, qdata, intvals, expnms, main=TRUE,
                       degree=degree, id=id,
-                      #weights=weights, 
-                      weights=NULL, 
+                      weights=weights, 
                       MCsize=MCsize, containmix=containmix, 
                       bayes=FALSE, x=FALSE, msmcontrol=msmcontrol, ...)
   #bootstrap to get std. error
@@ -543,10 +647,11 @@ qgcomp.zi.boot <- function(f,
   starttime = Sys.time()
   intvals = (1:length(table(qdata[expnms[1]]))) - 1
   Xpsi = rep(intvals, each=MCsize)
-  psi.only <- function(i=1, f=f, qdata=qdata, intvals=intvals, expnms=expnms, degree=degree, 
+  psi.only <- function(i=1, f=newform, qdata=qdata, intvals=intvals, expnms=expnms, degree=degree, 
                        nids=nids, id=id, 
-                       weights=NULL, 
+                       weights=weights,
                        ...){
+    
     if(i==2 & !parallel){
       timeiter = as.numeric(Sys.time() - starttime)
       if((timeiter*B/60)>0.5) message(paste0("Expected time to finish: ", round(B*timeiter/60, 2), " minutes \n"))
@@ -556,7 +661,7 @@ qgcomp.zi.boot <- function(f,
     qdata_ <- merge(qdata,bootids, by=id, all.x=FALSE, all.y=TRUE)
     ft = zimsm.fit(f, qdata_, intvals, expnms, main=FALSE,
                    degree=degree, id=id,
-                   weights=weights, 
+                   weights=weights,
                    MCsize=MCsize, containmix=containmix, 
                    bayes=FALSE, x=FALSE, msmcontrol=msmcontrol, ...)
     classprob = suppressWarnings(predict(ft$msmfit, type="prob"))
@@ -572,18 +677,17 @@ qgcomp.zi.boot <- function(f,
   if(parallel){
     Sys.setenv(R_FUTURE_SUPPORTSMULTICORE_UNSTABLE="quiet")
     future::plan(strategy = future::multiprocess)
-    bootsamps <- future.apply::future_sapply(X=1:B, FUN=psi.only,f=f, qdata=qdata, intvals=intvals, 
+    #testenv <- list2env(list(qdata=qdata, weights=weights))
+    bootsamps <- future.apply::future_sapply(X=1:B, FUN=psi.only,f=newform, qdata=qdata, intvals=intvals, 
                                              expnms=expnms, degree=degree, nids=nids, id=id, 
-                                             #weights=weights, 
-                                             weights=NULL, 
+                                             weights=qdata$weights,
                                              ...)
     
     future::plan(future::sequential)
   }else{
-    bootsamps <- sapply(X=1:B, FUN=psi.only,f=f, qdata=qdata, intvals=intvals, 
+    bootsamps <- sapply(X=1:B, FUN=psi.only,f=newform, qdata=qdata, intvals=intvals, 
                         expnms=expnms, degree=degree, nids=nids, id=id, 
-                        #weights=weights, 
-                        weights=NULL, 
+                        weights=weights, 
                         ...)
     
   }
